@@ -1,4 +1,4 @@
-import re
+import importlib
 
 from matplotlib import path, transforms
 from matplotlib.backend_bases import (
@@ -123,11 +123,21 @@ def test_location_event_position(x, y):
         assert event.y == int(y)
         assert isinstance(event.y, int)
     if x is not None and y is not None:
-        assert re.match(
-            f"x={ax.format_xdata(x)} +y={ax.format_ydata(y)}",
-            ax.format_coord(x, y))
+        assert (ax.format_coord(x, y)
+                == f"(x, y) = ({ax.format_xdata(x)}, {ax.format_ydata(y)})")
         ax.fmt_xdata = ax.fmt_ydata = lambda x: "foo"
-        assert re.match("x=foo +y=foo", ax.format_coord(x, y))
+        assert ax.format_coord(x, y) == "(x, y) = (foo, foo)"
+
+
+def test_location_event_position_twin():
+    fig, ax = plt.subplots()
+    ax.set(xlim=(0, 10), ylim=(0, 20))
+    assert ax.format_coord(5., 5.) == "(x, y) = (5.00, 5.00)"
+    ax.twinx().set(ylim=(0, 40))
+    assert ax.format_coord(5., 5.) == "(x, y) = (5.00, 5.00) | (5.00, 10.0)"
+    ax.twiny().set(xlim=(0, 5))
+    assert (ax.format_coord(5., 5.)
+            == "(x, y) = (5.00, 5.00) | (5.00, 10.0) | (2.50, 5.00)")
 
 
 def test_pick():
@@ -273,10 +283,11 @@ def test_toolbar_zoompan():
     with pytest.warns(UserWarning, match=_EXPECTED_WARNING_TOOLMANAGER):
         plt.rcParams['toolbar'] = 'toolmanager'
     ax = plt.gca()
+    fig = ax.get_figure()
     assert ax.get_navigate_mode() is None
-    ax.figure.canvas.manager.toolmanager.trigger_tool('zoom')
+    fig.canvas.manager.toolmanager.trigger_tool('zoom')
     assert ax.get_navigate_mode() == "ZOOM"
-    ax.figure.canvas.manager.toolmanager.trigger_tool('pan')
+    fig.canvas.manager.toolmanager.trigger_tool('pan')
     assert ax.get_navigate_mode() == "PAN"
 
 
@@ -317,9 +328,7 @@ def test_toolbar_home_restores_autoscale():
 def test_draw(backend):
     from matplotlib.figure import Figure
     from matplotlib.backends.backend_agg import FigureCanvas
-    test_backend = pytest.importorskip(
-        f'matplotlib.backends.backend_{backend}'
-    )
+    test_backend = importlib.import_module(f'matplotlib.backends.backend_{backend}')
     TestCanvas = test_backend.FigureCanvas
     fig_test = Figure(constrained_layout=True)
     TestCanvas(fig_test)
@@ -439,3 +448,136 @@ def test_toolmanager_update_keymap():
     assert fig.canvas.manager.toolmanager.get_tool_keymap('forward') == ['c']
     with pytest.raises(KeyError, match="'foo' not in Tools"):
         fig.canvas.manager.toolmanager.update_keymap('foo', 'c')
+
+
+@pytest.mark.parametrize("tool", ["zoom", "pan"])
+@pytest.mark.parametrize("button", [MouseButton.LEFT, MouseButton.RIGHT])
+@pytest.mark.parametrize("patch_vis", [True, False])
+@pytest.mark.parametrize("forward_nav", [True, False, "auto"])
+@pytest.mark.parametrize("t_s", ["twin", "share"])
+def test_interactive_pan_zoom_events(tool, button, patch_vis, forward_nav, t_s):
+    # Bottom axes: ax_b    Top axes: ax_t
+    fig, ax_b = plt.subplots()
+    ax_t = fig.add_subplot(221, zorder=99)
+    ax_t.set_forward_navigation_events(forward_nav)
+    ax_t.patch.set_visible(patch_vis)
+
+    # ----------------------------
+    if t_s == "share":
+        ax_t_twin = fig.add_subplot(222)
+        ax_t_twin.sharex(ax_t)
+        ax_t_twin.sharey(ax_t)
+
+        ax_b_twin = fig.add_subplot(223)
+        ax_b_twin.sharex(ax_b)
+        ax_b_twin.sharey(ax_b)
+    elif t_s == "twin":
+        ax_t_twin = ax_t.twinx()
+        ax_b_twin = ax_b.twinx()
+
+    # just some styling to simplify manual checks
+    ax_t.set_label("ax_t")
+    ax_t.patch.set_facecolor((1, 0, 0, 0.5))
+
+    ax_t_twin.set_label("ax_t_twin")
+    ax_t_twin.patch.set_facecolor("r")
+
+    ax_b.set_label("ax_b")
+    ax_b.patch.set_facecolor((0, 0, 1, 0.5))
+
+    ax_b_twin.set_label("ax_b_twin")
+    ax_b_twin.patch.set_facecolor("b")
+
+    # ----------------------------
+
+    # Set initial axis limits
+    init_xlim, init_ylim = (0, 10), (0, 10)
+    for ax in [ax_t, ax_b]:
+        ax.set_xlim(*init_xlim)
+        ax.set_ylim(*init_ylim)
+
+    # Mouse from 2 to 1 (in data-coordinates of ax_t).
+    xstart_t, xstop_t, ystart_t, ystop_t = 1, 2, 1, 2
+    # Convert to screen coordinates ("s").  Events are defined only with pixel
+    # precision, so round the pixel values, and below, check against the
+    # corresponding xdata/ydata, which are close but not equal to s0/s1.
+    s0 = ax_t.transData.transform((xstart_t, ystart_t)).astype(int)
+    s1 = ax_t.transData.transform((xstop_t, ystop_t)).astype(int)
+
+    # Calculate the mouse-distance in data-coordinates of the bottom-axes
+    xstart_b, ystart_b = ax_b.transData.inverted().transform(s0)
+    xstop_b, ystop_b = ax_b.transData.inverted().transform(s1)
+
+    # Set up the mouse movements
+    start_event = MouseEvent("button_press_event", fig.canvas, *s0, button)
+    stop_event = MouseEvent("button_release_event", fig.canvas, *s1, button)
+
+    tb = NavigationToolbar2(fig.canvas)
+
+    if tool == "zoom":
+        # Evaluate expected limits before executing the zoom-event
+        direction = ("in" if button == 1 else "out")
+
+        xlim_t, ylim_t = ax_t._prepare_view_from_bbox([*s0, *s1], direction)
+
+        if ax_t.get_forward_navigation_events() is True:
+            xlim_b, ylim_b = ax_b._prepare_view_from_bbox([*s0, *s1], direction)
+        elif ax_t.get_forward_navigation_events() is False:
+            xlim_b = init_xlim
+            ylim_b = init_ylim
+        else:
+            if not ax_t.patch.get_visible():
+                xlim_b, ylim_b = ax_b._prepare_view_from_bbox([*s0, *s1], direction)
+            else:
+                xlim_b = init_xlim
+                ylim_b = init_ylim
+
+        tb.zoom()
+        tb.press_zoom(start_event)
+        tb.drag_zoom(stop_event)
+        tb.release_zoom(stop_event)
+
+        assert ax_t.get_xlim() == pytest.approx(xlim_t, abs=0.15)
+        assert ax_t.get_ylim() == pytest.approx(ylim_t, abs=0.15)
+        assert ax_b.get_xlim() == pytest.approx(xlim_b, abs=0.15)
+        assert ax_b.get_ylim() == pytest.approx(ylim_b, abs=0.15)
+
+        # Check if twin-axes are properly triggered
+        assert ax_t.get_xlim() == pytest.approx(ax_t_twin.get_xlim(), abs=0.15)
+        assert ax_b.get_xlim() == pytest.approx(ax_b_twin.get_xlim(), abs=0.15)
+    else:
+        # Evaluate expected limits
+        # (call start_pan to make sure ax._pan_start is set)
+        ax_t.start_pan(*s0, button)
+        xlim_t, ylim_t = ax_t._get_pan_points(button, None, *s1).T.astype(float)
+        ax_t.end_pan()
+
+        if ax_t.get_forward_navigation_events() is True:
+            ax_b.start_pan(*s0, button)
+            xlim_b, ylim_b = ax_b._get_pan_points(button, None, *s1).T.astype(float)
+            ax_b.end_pan()
+        elif ax_t.get_forward_navigation_events() is False:
+            xlim_b = init_xlim
+            ylim_b = init_ylim
+        else:
+            if not ax_t.patch.get_visible():
+                ax_b.start_pan(*s0, button)
+                xlim_b, ylim_b = ax_b._get_pan_points(button, None, *s1).T.astype(float)
+                ax_b.end_pan()
+            else:
+                xlim_b = init_xlim
+                ylim_b = init_ylim
+
+        tb.pan()
+        tb.press_pan(start_event)
+        tb.drag_pan(stop_event)
+        tb.release_pan(stop_event)
+
+        assert ax_t.get_xlim() == pytest.approx(xlim_t, abs=0.15)
+        assert ax_t.get_ylim() == pytest.approx(ylim_t, abs=0.15)
+        assert ax_b.get_xlim() == pytest.approx(xlim_b, abs=0.15)
+        assert ax_b.get_ylim() == pytest.approx(ylim_b, abs=0.15)
+
+        # Check if twin-axes are properly triggered
+        assert ax_t.get_xlim() == pytest.approx(ax_t_twin.get_xlim(), abs=0.15)
+        assert ax_b.get_xlim() == pytest.approx(ax_b_twin.get_xlim(), abs=0.15)
