@@ -130,13 +130,14 @@ def test_tight_layout7():
     plt.tight_layout()
 
 
-@image_comparison(['tight_layout8'])
+@image_comparison(['tight_layout8'], tol=0.005)
 def test_tight_layout8():
     """Test automatic use of tight_layout."""
     fig = plt.figure()
-    fig.set_tight_layout({'pad': .1})
+    fig.set_layout_engine(layout='tight', pad=0.1)
     ax = fig.add_subplot()
     example_plot(ax, fontsize=24)
+    fig.draw_without_rendering()
 
 
 @image_comparison(['tight_layout9'])
@@ -172,13 +173,15 @@ def test_outward_ticks():
     plt.tight_layout()
     # These values were obtained after visual checking that they correspond
     # to a tight layouting that did take the ticks into account.
-    ans = [[[0.091, 0.607], [0.433, 0.933]],
-           [[0.579, 0.607], [0.922, 0.933]],
-           [[0.091, 0.140], [0.433, 0.466]],
-           [[0.579, 0.140], [0.922, 0.466]]]
+    expected = [
+        [[0.091, 0.607], [0.433, 0.933]],
+        [[0.579, 0.607], [0.922, 0.933]],
+        [[0.091, 0.140], [0.433, 0.466]],
+        [[0.579, 0.140], [0.922, 0.466]],
+    ]
     for nn, ax in enumerate(fig.axes):
         assert_array_equal(np.round(ax.get_position().get_points(), 3),
-                           ans[nn])
+                           expected[nn])
 
 
 def add_offsetboxes(ax, size=10, margin=.1, color='black'):
@@ -252,7 +255,7 @@ def test_tight_layout_offsetboxes():
 
 
 def test_empty_layout():
-    """Test that tight layout doesn't cause an error when there are no axes."""
+    """Test that tight layout doesn't cause an error when there are no Axes."""
     fig = plt.gcf()
     fig.tight_layout()
 
@@ -329,3 +332,64 @@ def test_non_agg_renderer(monkeypatch, recwarn):
     monkeypatch.setattr(mpl.backend_bases.RendererBase, "__init__", __init__)
     fig, ax = plt.subplots()
     fig.tight_layout()
+
+
+def test_manual_colorbar():
+    # This should warn, but not raise
+    fig, axes = plt.subplots(1, 2)
+    pts = axes[1].scatter([0, 1], [0, 1], c=[1, 5])
+    ax_rect = axes[1].get_position()
+    cax = fig.add_axes(
+        [ax_rect.x1 + 0.005, ax_rect.y0, 0.015, ax_rect.height]
+    )
+    fig.colorbar(pts, cax=cax)
+    with pytest.warns(UserWarning, match="This figure includes Axes"):
+        fig.tight_layout()
+
+
+def test_clipped_to_axes():
+    # Ensure that _fully_clipped_to_axes() returns True under default
+    # conditions for all projection types. Axes.get_tightbbox()
+    # uses this to skip artists in layout calculations.
+    arr = np.arange(100).reshape((10, 10))
+    fig = plt.figure(figsize=(6, 2))
+    ax1 = fig.add_subplot(131, projection='rectilinear')
+    ax2 = fig.add_subplot(132, projection='mollweide')
+    ax3 = fig.add_subplot(133, projection='polar')
+    for ax in (ax1, ax2, ax3):
+        # Default conditions (clipped by ax.bbox or ax.patch)
+        ax.grid(False)
+        h, = ax.plot(arr[:, 0])
+        m = ax.pcolor(arr)
+        assert h._fully_clipped_to_axes()
+        assert m._fully_clipped_to_axes()
+        # Non-default conditions (not clipped by ax.patch)
+        rect = Rectangle((0, 0), 0.5, 0.5, transform=ax.transAxes)
+        h.set_clip_path(rect)
+        m.set_clip_path(rect.get_path(), rect.get_transform())
+        assert not h._fully_clipped_to_axes()
+        assert not m._fully_clipped_to_axes()
+
+
+def test_tight_pads():
+    fig, ax = plt.subplots()
+    with pytest.warns(PendingDeprecationWarning,
+                      match='will be deprecated'):
+        fig.set_tight_layout({'pad': 0.15})
+    fig.draw_without_rendering()
+
+
+def test_tight_kwargs():
+    fig, ax = plt.subplots(tight_layout={'pad': 0.15})
+    fig.draw_without_rendering()
+
+
+def test_tight_toggle():
+    fig, ax = plt.subplots()
+    with pytest.warns(PendingDeprecationWarning):
+        fig.set_tight_layout(True)
+        assert fig.get_tight_layout()
+        fig.set_tight_layout(False)
+        assert not fig.get_tight_layout()
+        fig.set_tight_layout(True)
+        assert fig.get_tight_layout()
